@@ -4,10 +4,11 @@ import functools
 import multiprocessing
 import traceback
 from concurrent.futures import Future, ThreadPoolExecutor
-from typing import Any, Callable, Collection, Dict, List, Optional, Tuple, Union
+from typing import TYPE_CHECKING, Any, Callable, Collection, TypeVar
+
+from rich.progress import SpinnerColumn
 
 from pdm import termui
-from pdm._types import Distribution
 from pdm.exceptions import InstallationError
 from pdm.installers.manager import InstallManager
 from pdm.models.candidates import Candidate
@@ -15,13 +16,21 @@ from pdm.models.environment import Environment
 from pdm.models.requirements import parse_requirement, strip_extras
 from pdm.utils import is_editable
 
+if TYPE_CHECKING:
+    from rich.progress import Progress
+
+    from pdm.compat import Distribution
+
+
+_T = TypeVar("_T")
+
 
 class DummyFuture:
     _NOT_SET = object()
 
     def __init__(self) -> None:
         self._result = self._NOT_SET
-        self._exc: Optional[Exception] = None
+        self._exc: Exception | None = None
 
     def set_result(self, result: Any) -> None:
         self._result = result
@@ -32,10 +41,10 @@ class DummyFuture:
     def result(self) -> Any:
         return self._result
 
-    def exception(self) -> Optional[Exception]:
+    def exception(self) -> Exception | None:
         return self._exc
 
-    def add_done_callback(self, func: Callable) -> None:
+    def add_done_callback(self: _T, func: Callable[[_T], Any]) -> None:
         func(self)
 
 
@@ -44,7 +53,7 @@ class DummyExecutor:
     functions are called and awaited for the result
     """
 
-    def submit(self, func: Callable, *args: str, **kwargs: Any) -> DummyFuture:
+    def submit(self, func: Callable, *args: Any, **kwargs: Any) -> DummyFuture:
         future = DummyFuture()
         try:
             future.set_result(func(*args, **kwargs))
@@ -52,7 +61,7 @@ class DummyExecutor:
             future.set_exception(exc)
         return future
 
-    def __enter__(self) -> DummyExecutor:
+    def __enter__(self: _T) -> _T:
         return self
 
     def __exit__(self, *args: Any, **kwargs: Any) -> None:
@@ -78,13 +87,14 @@ class Synchronizer:
         if a list, override editables with the given names
     :param use_install_cache: whether to use install cache
     :param reinstall: whether to reinstall all packages
+    :param only_keep: If true, only keep the selected candidates
     """
 
     SEQUENTIAL_PACKAGES = ("pip", "setuptools", "wheel")
 
     def __init__(
         self,
-        candidates: Dict[str, Candidate],
+        candidates: dict[str, Candidate],
         environment: Environment,
         clean: bool = False,
         dry_run: bool = False,
@@ -93,6 +103,7 @@ class Synchronizer:
         no_editable: bool | Collection[str] = False,
         use_install_cache: bool = False,
         reinstall: bool = False,
+        only_keep: bool = False,
     ) -> None:
         self.environment = environment
         self.clean = clean
@@ -102,6 +113,7 @@ class Synchronizer:
         self.install_self = install_self
         self.use_install_cache = use_install_cache
         self.reinstall = reinstall
+        self.only_keep = only_keep
 
         self.parallel = environment.project.config["install.parallel"]
         locked_repository = environment.project.locked_repository
@@ -117,7 +129,9 @@ class Synchronizer:
             keys = []
             if (
                 self.install_self
-                and getattr(self.environment.project.meta, "editable_backend", "path")
+                and getattr(
+                    self.environment.project.meta.config, "editable_backend", "path"
+                )
                 == "editables"
                 and "editables" not in candidates
             ):
@@ -133,7 +147,7 @@ class Synchronizer:
 
     def create_executor(
         self,
-    ) -> Union[ThreadPoolExecutor, DummyExecutor]:
+    ) -> ThreadPoolExecutor | DummyExecutor:
         if self.parallel:
             return ThreadPoolExecutor(max_workers=min(multiprocessing.cpu_count(), 8))
         else:
@@ -151,7 +165,7 @@ class Synchronizer:
         )
 
     @property
-    def self_key(self) -> Optional[str]:
+    def self_key(self) -> str | None:
         name = self.environment.project.name
         if name:
             return self.environment.project.meta.project_name.lower()
@@ -166,11 +180,12 @@ class Synchronizer:
         else:
             return dist.version != can.version
 
-    def compare_with_working_set(self) -> Tuple[List[str], List[str], List[str]]:
+    def compare_with_working_set(self) -> tuple[list[str], list[str], list[str]]:
         """Compares the candidates and return (to_add, to_update, to_remove)"""
         working_set = self.working_set
         candidates = self.candidates.copy()
-        to_update, to_remove = [], []
+        to_update: set[str] = set()
+        to_remove: set[str] = set()
 
         for key, dist in working_set.items():
             if key == self.self_key:
@@ -178,106 +193,119 @@ class Synchronizer:
             if key in candidates:
                 can = candidates.pop(key)
                 if self._should_update(dist, can):
-                    to_update.append(key)
+                    to_update.add(key)
             elif (
-                key not in self.all_candidate_keys
-                and key not in self.SEQUENTIAL_PACKAGES
-            ):
+                self.only_keep or self.clean and key not in self.all_candidate_keys
+            ) and key not in self.SEQUENTIAL_PACKAGES:
                 # Remove package only if it is not required by any group
                 # Packages for packaging will never be removed
-                to_remove.append(key)
-        to_add = list(
-            {
-                strip_extras(name)[0]
-                for name, _ in candidates.items()
-                if name != self.self_key and strip_extras(name)[0] not in working_set
-            }
-        )
+                to_remove.add(key)
+        to_add = {
+            strip_extras(name)[0]
+            for name, _ in candidates.items()
+            if name != self.self_key and strip_extras(name)[0] not in working_set
+        }
         return (
             sorted(to_add),
             sorted(to_update),
-            sorted(to_remove) if self.clean else [],
+            sorted(to_remove),
         )
 
-    def install_candidate(self, key: str) -> Candidate:
+    def install_candidate(self, key: str, progress: Progress) -> Candidate:
         """Install candidate"""
         can = self.candidates[key]
-        with self.ui.open_spinner(f"Installing {can.format()}...") as spinner:
-            try:
-                self.manager.install(can)
-            except Exception:
-                spinner.fail(f"Install {can.format()} failed")
-                raise
-            else:
-                spinner.succeed(f"Install {can.format()} successful")
-
+        job = progress.add_task(f"Installing {can.format()}...", total=1)
+        try:
+            self.manager.install(can)
+        except Exception:
+            progress.live.console.print(
+                f"  [red]{termui.Emoji.FAIL}[/] Install {can.format()} failed"
+            )
+            raise
+        else:
+            progress.live.console.print(
+                f"  [green]{termui.Emoji.SUCC}[/] Install {can.format()} successful"
+            )
+        finally:
+            progress.update(job, completed=1, visible=False)
         return can
 
-    def update_candidate(self, key: str) -> Tuple[Distribution, Candidate]:
+    def update_candidate(
+        self, key: str, progress: Progress
+    ) -> tuple[Distribution, Candidate]:
         """Update candidate"""
         can = self.candidates[key]
         dist = self.working_set[strip_extras(key)[0]]
         dist_version = dist.version
-        with self.ui.open_spinner(
-            f"Updating {termui.green(key, bold=True)} {termui.yellow(dist.version)} "
-            f"-> {termui.yellow(can.version)}..."
-        ) as spinner:
-            try:
-                self.manager.uninstall(dist)
-                self.manager.install(can)
-            except Exception:
-                spinner.fail(
-                    f"Update {termui.green(key, bold=True)} "
-                    f"{termui.yellow(dist_version)} -> "
-                    f"{termui.yellow(can.version)} failed"
-                )
-                raise
-            else:
-                spinner.succeed(
-                    f"Update {termui.green(key, bold=True)} "
-                    f"{termui.yellow(dist_version)} -> "
-                    f"{termui.yellow(can.version)} successful"
-                )
+        job = progress.add_task(
+            f"Updating [bold green]{key}[/] "
+            f"[yellow]{dist_version}[/] "
+            f"-> [yellow]{can.version}[/]...",
+            total=1,
+        )
+        try:
+            self.manager.uninstall(dist)
+            self.manager.install(can)
+        except Exception:
+            progress.live.console.print(
+                f"  [red]{termui.Emoji.FAIL}[/] Update [bold green]{key}[/] "
+                f"[yellow]{dist_version}[/] "
+                f"-> [yellow]{can.version}[/] failed",
+            )
+            raise
+        else:
+            progress.live.console.print(
+                f"  [green]{termui.Emoji.SUCC}[/] Update [bold green]{key}[/] "
+                f"[yellow]{dist_version}[/] "
+                f"-> [yellow]{can.version}[/] successful",
+            )
+        finally:
+            progress.update(job, completed=1, visible=False)
+
         return dist, can
 
-    def remove_distribution(self, key: str) -> Distribution:
+    def remove_distribution(self, key: str, progress: Progress) -> Distribution:
         """Remove distributions with given names."""
         dist = self.working_set[key]
         dist_version = dist.version
-        with self.ui.open_spinner(
-            f"Removing {termui.green(key, bold=True)} {termui.yellow(dist.version)}..."
-        ) as spinner:
-            try:
-                self.manager.uninstall(dist)
-            except Exception:
-                spinner.fail(
-                    f"Remove {termui.green(key, bold=True)} "
-                    f"{termui.yellow(dist_version)} failed"
-                )
-                raise
-            else:
-                spinner.succeed(
-                    f"Remove {termui.green(key, bold=True)} "
-                    f"{termui.yellow(dist_version)} successful"
-                )
+
+        job = progress.add_task(
+            f"Removing [bold green]{key}[/] " f"[yellow]{dist_version}[/]...",
+            total=1,
+        )
+        try:
+            self.manager.uninstall(dist)
+        except Exception:
+            progress.live.console.print(
+                f"  [red]{termui.Emoji.FAIL}[/] Remove [bold green]{key}[/] "
+                f"[yellow]{dist_version}[/] failed",
+            )
+            raise
+        else:
+            progress.live.console.print(
+                f"  [green]{termui.Emoji.SUCC}[/] Remove [bold green]{key}[/] "
+                f"[yellow]{dist_version}[/] successful"
+            )
+        finally:
+            progress.update(job, completed=1, visible=False)
         return dist
 
-    def _show_headline(self, packages: Dict[str, List[str]]) -> None:
+    def _show_headline(self, packages: dict[str, list[str]]) -> None:
         add, update, remove = packages["add"], packages["update"], packages["remove"]
         if not any((add, update, remove)):
-            self.ui.echo("All packages are synced to date, nothing to do.\n")
+            self.ui.echo("All packages are synced to date, nothing to do.")
             return
-        results = [termui.bold("Synchronizing working set with lock file:")]
+        results = ["[bold]Synchronizing working set with lock file[/]:"]
         results.extend(
             [
-                f"{termui.green(str(len(add)))} to add,",
-                f"{termui.yellow(str(len(update)))} to update,",
-                f"{termui.red(str(len(remove)))} to remove",
+                f"[green]{len(add)}[/] to add,",
+                f"[yellow]{len(update)}[/] to update,",
+                f"[red]{len(remove)}[/] to remove",
             ]
         )
         self.ui.echo(" ".join(results) + "\n")
 
-    def _show_summary(self, packages: Dict[str, List[str]]) -> None:
+    def _show_summary(self, packages: dict[str, list[str]]) -> None:
         to_add = [self.candidates[key] for key in packages["add"]]
         to_update = [
             (self.working_set[key], self.candidates[key]) for key in packages["update"]
@@ -285,22 +313,22 @@ class Synchronizer:
         to_remove = [self.working_set[key] for key in packages["remove"]]
         lines = []
         if to_add:
-            lines.append(termui.bold("Packages to add:"))
+            lines.append("[bold]Packages to add[/]:")
             for can in to_add:
                 lines.append(f"  - {can.format()}")
         if to_update:
-            lines.append(termui.bold("Packages to update:"))
+            lines.append("[bold]Packages to update[/]:")
             for prev, cur in to_update:
                 lines.append(
-                    f"  - {termui.green(cur.name, bold=True)} "
-                    f"{termui.yellow(prev.version)} -> {termui.yellow(cur.version)}"
+                    f"  - [bold green]{cur.name}[/] "
+                    f"[yellow]{prev.version}[/] -> [yellow]{cur.version}[/]"
                 )
         if to_remove:
-            lines.append(termui.bold("Packages to remove:"))
+            lines.append("[bold]Packages to remove[/]:")
             for dist in to_remove:
                 lines.append(
-                    f"  - {termui.green(dist.metadata['Name'], bold=True)} "
-                    f"{termui.yellow(dist.version)}"
+                    f"  - [bold green]{dist.metadata['Name']}[/] "
+                    f"[yellow]{dist.version}[/]"
                 )
         if lines:
             self.ui.echo("\n".join(lines))
@@ -333,44 +361,46 @@ class Synchronizer:
                 else:
                     parallel_jobs.append((kind, key))
 
-        errors: List[str] = []
-        failed_jobs: List[Tuple[str, str]] = []
+        errors: list[str] = []
+        failed_jobs: list[tuple[str, str]] = []
 
-        def update_progress(
-            future: Union[Future, DummyFuture], kind: str, key: str
-        ) -> None:
+        def update_progress(future: Future | DummyFuture, kind: str, key: str) -> None:
             error = future.exception()
             if error:
                 exc_info = (type(error), error, error.__traceback__)
                 termui.logger.exception("Error occurs: ", exc_info=exc_info)
                 failed_jobs.append((kind, key))
                 errors.extend(
-                    [f"{kind} {termui.green(key)} failed:\n"]
+                    [f"{kind} [green]{key}[/] failed:\n"]
                     + traceback.format_exception(*exc_info)
                 )
 
-        with self.ui.logging("install"):
-            with self.ui.indent("  "):
-                for job in sequential_jobs:
-                    kind, key = job
-                    handlers[kind](key)
-                for i in range(self.retry_times + 1):
-                    with self.create_executor() as executor:
-                        for job in parallel_jobs:
-                            kind, key = job
-                            future = executor.submit(handlers[kind], key)
-                            future.add_done_callback(
-                                functools.partial(update_progress, kind=kind, key=key)
-                            )
-                    if not failed_jobs or i == self.retry_times:
-                        break
-                    parallel_jobs, failed_jobs = failed_jobs, []
-                    errors.clear()
-                    self.ui.echo("Retry failed jobs")
+        # get rich progress and live handler to deal with multiple spinners
+        with self.ui.logging("install"), self.ui.make_progress(
+            " ",
+            SpinnerColumn(termui.SPINNER, speed=1, style="bold cyan"),
+            "{task.description}",
+        ) as progress:
+            live = progress.live
+            for kind, key in sequential_jobs:
+                handlers[kind](key, progress)
+            for i in range(self.retry_times + 1):
+                with self.create_executor() as executor:
+                    for kind, key in parallel_jobs:
+                        future = executor.submit(handlers[kind], key, progress)
+                        future.add_done_callback(
+                            functools.partial(update_progress, kind=kind, key=key)
+                        )
+                if not failed_jobs or i == self.retry_times:
+                    break
+                parallel_jobs, failed_jobs = failed_jobs, []
+                errors.clear()
+                live.console.print("Retry failed jobs")
 
             if errors:
-                self.ui.echo(termui.red("\nERRORS:"))
-                self.ui.echo("".join(errors), err=True)
+                if self.ui.verbosity < termui.Verbosity.DETAIL:
+                    live.console.print("\n[red]ERRORS[/]:")
+                    live.console.print("".join(errors), end="")
                 raise InstallationError("Some package operations are not complete yet")
 
             if self.install_self:
@@ -381,11 +411,10 @@ class Synchronizer:
                 assert self_key
                 self.candidates[self_key] = self_candidate
                 word = "a" if self.no_editable else "an editable"
-                self.ui.echo(f"Installing the project as {word} package...")
-                with self.ui.indent("  "):
-                    if self_key in self.working_set:
-                        self.update_candidate(self_key)
-                    else:
-                        self.install_candidate(self_key)
+                live.console.print(f"Installing the project as {word} package...")
+                if self_key in self.working_set:
+                    self.update_candidate(self_key, progress)
+                else:
+                    self.install_candidate(self_key, progress)
 
-            self.ui.echo(f"\n{termui.Emoji.SUCC} All complete!")
+            live.console.print(f"\n{termui.Emoji.POPPER} All complete!")
