@@ -1,11 +1,11 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Callable, cast
 
 from packaging.specifiers import InvalidSpecifier, SpecifierSet
 from resolvelib import AbstractProvider
 
-from pdm.models.candidates import Candidate
+from pdm.models.candidates import Candidate, make_candidate
 from pdm.models.requirements import parse_requirement, strip_extras
 from pdm.resolver.python import (
     PythonCandidate,
@@ -34,7 +34,7 @@ class BaseProvider(AbstractProvider):
     ) -> None:
         self.repository = repository
         self.allow_prereleases = allow_prereleases  # Root allow_prereleases value
-        self.fetched_dependencies: dict[str, list[Requirement]] = {}
+        self.fetched_dependencies: dict[tuple[str, str | None], list[Requirement]] = {}
         self.overrides = overrides or {}
         self._known_depth: dict[str, int] = {}
 
@@ -65,7 +65,7 @@ class BaseProvider(AbstractProvider):
         candidates: dict[str, Iterator[Candidate]],
         information: dict[str, Iterator[RequirementInformation]],
         backtrack_causes: Sequence[RequirementInformation],
-    ) -> Comparable:
+    ) -> tuple[Comparable, ...]:
         is_top = any(parent is None for _, parent in information[identifier])
         is_backtrack_cause = any(
             requirement.identify() == identifier
@@ -121,8 +121,8 @@ class BaseProvider(AbstractProvider):
 
     def _find_candidates(self, requirement: Requirement) -> Iterable[Candidate]:
         if not requirement.is_named:
-            can = Candidate(requirement)
-            can.prepare(self.repository.environment).prepare_metadata()
+            can = make_candidate(requirement)
+            can.prepare(self.repository.environment).metadata
             return [can]
         else:
             return self.repository.find_candidates(
@@ -134,20 +134,24 @@ class BaseProvider(AbstractProvider):
         identifier: str,
         requirements: Mapping[str, Iterator[Requirement]],
         incompatibilities: Mapping[str, Iterator[Candidate]],
-    ) -> Iterable[Candidate]:
-        incompat = list(incompatibilities[identifier])
-        if identifier == "python":
-            candidates = find_python_matches(identifier, requirements)
-            return [c for c in candidates if c not in incompat]
-        elif identifier in self.overrides:
-            return self.get_override_candidates(identifier)
-        reqs = sorted(requirements[identifier], key=self.requirement_preference)
-        candidates = self._find_candidates(reqs[0])
-        return [
-            can
-            for can in candidates
-            if can not in incompat and all(self.is_satisfied_by(r, can) for r in reqs)
-        ]
+    ) -> Callable[[], Iterator[Candidate]]:
+        def matches_gen() -> Iterator[Candidate]:
+            incompat = list(incompatibilities[identifier])
+            if identifier == "python":
+                candidates = find_python_matches(identifier, requirements)
+                return (c for c in candidates if c not in incompat)
+            elif identifier in self.overrides:
+                return iter(self.get_override_candidates(identifier))
+            reqs = sorted(requirements[identifier], key=self.requirement_preference)
+            candidates = self._find_candidates(reqs[0])
+            return (
+                can
+                for can in candidates
+                if can not in incompat
+                and all(self.is_satisfied_by(r, can) for r in reqs)
+            )
+
+        return matches_gen
 
     def is_satisfied_by(self, requirement: Requirement, candidate: Candidate) -> bool:
         if isinstance(requirement, PythonRequirement):
@@ -156,15 +160,19 @@ class BaseProvider(AbstractProvider):
             return True
         if not requirement.is_named:
             return not candidate.req.is_named and url_without_fragments(
-                candidate.req.url
-            ) == url_without_fragments(requirement.url)
-        version = candidate.version or candidate.metadata.version
+                candidate.req.url  # type: ignore
+            ) == url_without_fragments(
+                requirement.url  # type: ignore
+            )
+        version = candidate.version
         # Allow prereleases if: 1) it is not specified in the tool settings or
         # 2) the candidate doesn't come from PyPI index.
         allow_prereleases = (
             self.allow_prereleases in (True, None) or not candidate.req.is_named
         )
-        return requirement.specifier.contains(version, allow_prereleases)
+        return cast(SpecifierSet, requirement.specifier).contains(
+            version, allow_prereleases
+        )
 
     def get_dependencies(self, candidate: Candidate) -> list[Requirement]:
         if isinstance(candidate, PythonCandidate):
@@ -184,7 +192,7 @@ class BaseProvider(AbstractProvider):
                 continue
             dep.requires_python &= candidate.req.requires_python
             valid_deps.append(dep)
-        self.fetched_dependencies[self.identify(candidate)] = valid_deps[:]
+        self.fetched_dependencies[candidate.dep_key] = valid_deps[:]
         # A candidate contributes to the Python requirements only when:
         # It isn't an optional dependency, or the requires-python doesn't cover
         # the req's requires-python.
@@ -222,20 +230,28 @@ class ReusePinProvider(BaseProvider):
         identifier: str,
         requirements: Mapping[str, Iterator[Requirement]],
         incompatibilities: Mapping[str, Iterator[Candidate]],
-    ) -> Iterable[Candidate]:
+    ) -> Callable[[], Iterator[Candidate]]:
+        super_find = super().find_matches(identifier, requirements, incompatibilities)
         bare_name = strip_extras(identifier)[0]
-        if bare_name not in self.tracked_names and identifier in self.preferred_pins:
-            pin = self.preferred_pins[identifier]
-            incompat = list(incompatibilities[identifier])
-            demanded_req = next(requirements[identifier], None)
-            if demanded_req and demanded_req.is_named:
-                pin.req = demanded_req
-            pin._preferred = True
-            if pin not in incompat and all(
-                self.is_satisfied_by(r, pin) for r in requirements[identifier]
+
+        def matches_gen() -> Iterator[Candidate]:
+            if (
+                bare_name not in self.tracked_names
+                and identifier in self.preferred_pins
             ):
-                yield pin
-        yield from super().find_matches(identifier, requirements, incompatibilities)
+                pin = self.preferred_pins[identifier]
+                incompat = list(incompatibilities[identifier])
+                demanded_req = next(requirements[identifier], None)
+                if demanded_req and demanded_req.is_named:
+                    pin.req = demanded_req
+                pin._preferred = True  # type: ignore
+                if pin not in incompat and all(
+                    self.is_satisfied_by(r, pin) for r in requirements[identifier]
+                ):
+                    yield pin
+            yield from super_find()
+
+        return matches_gen
 
 
 class EagerUpdateProvider(ReusePinProvider):
@@ -276,7 +292,7 @@ class EagerUpdateProvider(ReusePinProvider):
         candidates: dict[str, Iterator[Candidate]],
         information: dict[str, Iterator[RequirementInformation]],
         backtrack_causes: Sequence[RequirementInformation],
-    ) -> Comparable:
+    ) -> tuple[Comparable, ...]:
         # Resolve tracking packages so we have a chance to unpin them first.
         (python, *others) = super().get_preference(
             identifier, resolutions, candidates, information, backtrack_causes
